#include <pic16f876a.h>
#include <string.h>

#include "commandes.h"
#include "commun.h"
#include "serial.h"

void decoderCommande(){
    /*
     * TEST\r => OK\n\r
     * START\r => OK\n\r
     * END\r => OK\n\r
     * WRITE<adressHigh><adressLow><longueur>\r<octet0><octet1><octet2>...<octetN> => OK\n\r ou ERR\n\r
     * READ<adressHighDebut><adressLowDebut><adressHighFin><adressLowFin>\r => VALUE<data>\n\r ou ERR\n\r
     * WRITEDELAY<delayHigh><delayLow>\r => OK\n\r ou ERR\n\r
     * 
     * TODO:
     * MODEMEM<modeMemoire>\n => OK\r\r ou ERR\n\r
     * CHGBANK<blocMemoire><bank>\r => OK\n\r ou ERR\n\r
     * 
     * ERR: Doit faire START avant pour prendre controle sur les bus
    */

    // Commande de test
    if ( verifCommande("TEST\r") > 0 ){
        commandeTEST();
    }
    else if ( verifCommande("INFO\r") > 0 ){
        commandeINFO();
    }
    else if ( verifCommande("START\r") > 0 ){
        commandeSTART();
    }
    else if ( verifCommande("END\r") > 0 ){
        commandeEND();
    }
    else if ( verifCommande("WRITE***\r") > 0 ){
        commandeWRITE();
    }
    else if ( verifCommande("READ****\r") > 0 ){
        commandeREAD();
    }   
    else if ( verifCommande("WRITEDELAY**\r") > 0 ){
        commandeWRITEDELAY();
    }
    else{
        envoiString("UNKNOWN_COMMAND\n\r");
    }
}

char verifCommande(char commandeAttendue[]){
    char result = 1;
    for ( char i = 0 ; i < strlen(commandeAttendue) ; i++ ){
        if ( buffer1[(BUFFER_1_SIZE - strlen(commandeAttendue)) + i] != commandeAttendue[i] && commandeAttendue[i] != '*' ){
            result = 0;
            break;
        }
    }
    return result;
}

void commandeTEST(){
    // Commande de test
    envoiString("OK\n\r");
}

void commandeINFO(){
    // Informations sur le circuit de developpement
    envoiString("OrdiZ80-CircuitDev-1\n\r");
}

void commandeSTART(){
    // Début de la séquence de programmation
    // Mise en RESET du CPU et prise de controle des bus
    if ( progEnCours == 0 ){
        progEnCours = 1;

        // Reset valeurs
        setAdress(0, 0);
        setData(0);
        controlBus.RESET = 0;
        setControl(controlBus);

        // Activer bus
        busActifs.adressEtControl = 1;
        busActifs.sortieData = 1;
        busActifs.entreeData = 0;
        mettreAJourBusActifs();

        envoiString("OK\n\r");
    }
    else{
        envoiString("ERR\n\r");
    }
}

void commandeEND(){
    // Fin de la programmation
    // Arrêt de la prise des controle des bus, le CPU redémarre
    if ( progEnCours == 1 ){
        progEnCours = 0;
                
        busActifs.adressEtControl = 0;
        busActifs.sortieData = 0;
        busActifs.entreeData = 0;
        mettreAJourBusActifs();

        controlBus.RESET = 1;
        setControl(controlBus);

        envoiString("OK\n\r");
    }
    else{
        envoiString("ERR\n\r");
    }
}

void commandeWRITE(){
    // Ecriture d'une ensemble d'octets en mémoire
    // WRITE<adressHigh><adressLow><longueur>\r<octet0><octet1><octet2>...<octetN>
    if ( progEnCours == 1 ){
        unsigned char adressHigh = buffer1[BUFFER_1_SIZE - 4];
        unsigned char adressLow = buffer1[BUFFER_1_SIZE - 3];
        unsigned int longueur = buffer1[BUFFER_1_SIZE - 2] + 1;

        envoiString("SEND\n\r");

        if ( longueur > 0 && longueur <= (BUFFER_1_SIZE + BUFFER_2_SIZE + BUFFER_3_SIZE) ){
                            
            // Réception de l'ensemble d'octets
            unsigned int i;
            for ( i = 0 ; i < longueur ; i++ ){
                if ( RCSTAbits.OERR ){
                    RCSTAbits.CREN = 0;
                    RCSTAbits.CREN = 1;
                }
                while ( !PIR1bits.RCIF );
                if ( i < 96 ){
                    buffer1[i] = RCREG;
                }
                else if ( i >= 96 && i < 192 ){
                    buffer2[i - 96] = RCREG;
                }
                else{
                    buffer3[i - 192] = RCREG;
                }
            }
                        
            // Octets reçus
            // Ecriture en mémoire à la position demandée
            controlBus.MREQ = 0;
            setControl(controlBus);
                    
            setAdressHigh(adressHigh);
            for ( i = 0 ; i < longueur ; i++ ){
                //setAdress(adressLow, adressHigh);
                setAdressLow(adressLow);
                        
                if ( i < 96 ){
                    setData(buffer1[i]);
                }
                else if ( i >= 96 && i < 192 ){
                    setData(buffer2[i - 96]);
                }
                else{
                    setData(buffer3[i - 192]);
                }

                controlBus.WR = 0;
                setControl(controlBus);

                // boucle d'attente écriture
                for ( int i = 0 ; i < temporisationEcriture ; i++ );
                        
                controlBus.WR = 1;
                setControl(controlBus);
                            
                for ( int i = 0 ; i < temporisationEcriture ; i++ );
                        
                adressLow++;
                if ( adressLow == 0 ){
                    adressHigh++;
                    setAdressHigh(adressHigh);
                }
            }
            setAdress(0, 0);
            setData(0);
            controlBus.MREQ = 1;
            setControl(controlBus);
                        
            // Effacement des buffers
            for ( i = 0 ; i < BUFFER_1_SIZE ; i++ ){
                buffer1[i] = 0;
            }
            for ( i = 0 ; i < BUFFER_2_SIZE ; i++ ){
                buffer2[i] = 0;
            }
            for ( i = 0 ; i < BUFFER_3_SIZE ; i++ ){
                buffer3[i] = 0;
            }
            envoiString("OK\n\r");
        }
        else{
            envoiString("BUFFER_ERR\n\r");
        }
    }
    else{
        envoiString("ERR\n\r");
    }
}

void commandeREAD(){
    // Lecture d'un ensemble d'octets stocké en mémoire
    // READ<adressHighDebut><adressLowDebut><adressHighFin><adressLowFin>\r
    if ( progEnCours == 1 ){
        unsigned char adressHighDebut = buffer1[BUFFER_1_SIZE - 5];
        unsigned char adressLowDebut = buffer1[BUFFER_1_SIZE - 4];
        unsigned char adressHighFin = buffer1[BUFFER_1_SIZE - 3];
        unsigned char adressLowFin = buffer1[BUFFER_1_SIZE - 2];
                    
        envoiString("VALUES");
                    
        busActifs.sortieData = 0;
        mettreAJourBusActifs();
        while ( adressLowDebut <= 255 ){
                        
            setAdress(adressLowDebut, adressHighDebut);
            controlBus.MREQ = 0;
            controlBus.RD = 0;
            setControl(controlBus);

            TRISB = 0b11111111;

            busActifs.entreeData = 1;
            mettreAJourBusActifs();

            envoi(getData());

            busActifs.entreeData = 0;
            mettreAJourBusActifs();

            TRISB = 0b00000000;
                        
            controlBus.MREQ = 1;
            controlBus.RD = 1;
            setControl(controlBus);
                        
            if ( adressLowDebut == adressLowFin && adressHighDebut == adressHighFin ){
                break;
            }
                        
            if ( adressLowDebut < 255 ){
                adressLowDebut++;
            }
            else {
                adressLowDebut = 0;
                if ( adressHighDebut < 255 ){
                    adressHighDebut++;
                }
                else{
                    break;
                }
            }
        }
        busActifs.sortieData = 1;
        mettreAJourBusActifs();
        
        envoiString("\n\r");
    }
    else{
        envoiString("ERR\n\r");
    }
}

void commandeWRITEDELAY(){
    // Défintion de delais d'attendre d'écriture pour chaque octet
    // WRITEDELAY<delayHigh><delayLow>\r
    if ( progEnCours == 1 ){
        unsigned char delayHigh = buffer1[BUFFER_1_SIZE - 3];
        unsigned char delayLow = buffer1[BUFFER_1_SIZE - 2];
                
        temporisationEcriture = delayHigh*256 + delayLow;
        
        envoiString("OK\n\r");
    }
    else{
        envoiString("ERR\n\r");
    }
}