/* 
 * File:   main.c
 * Author: Jelora
 *
 * Created on 18 novembre 2018, 01:52
 */

// CONFIG
#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable bit (BOR disabled)
#pragma config LVP = OFF        // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pic16f876a.h>
#include <xc.h>

#include "commun.h"
#include "serial.h"
#include "commandes.h"


int main(int argc, char** argv) {
    
    // Port A avec IO numérique
    ADCON1 = 0b000000110;
    
    // Config IO
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b10000000;

    // Config liaison série
    //SPBRG = 25; // 9600 bits/s
    //SPBRG = 12; // 19200 bits/s
    SPBRG = 4; // 230400 bits/s
    TXSTA = 0b00100100;
    RCSTA = 0b10010000;
    
    // Valeurs au démarrage
    PORTA = 0b00000111;
    PORTB = 0b00000000;
    PORTC = 0b00000000;
    
    busActifs.adressEtControl = 0;
    busActifs.sortieData = 0;
    busActifs.entreeData = 0;
    
    controlBus.M1 = 1;
    controlBus.MREQ = 1;
    controlBus.IOREQ = 1;
    controlBus.RD = 1;
    controlBus.WR = 1;
    controlBus.RFSH = 1;
    controlBus.HALT = 1;
    controlBus.WAIT = 1;
    controlBus.INT = 1;
    controlBus.NMI = 1;
    controlBus.RESET = 1;
    controlBus.BUSRQ = 1;
    controlBus.BUSAK = 1;
    
    while ( 1 ){
        // Reset de l'overflow sur le buffer du port série
        if ( RCSTAbits.OERR ){
            RCSTAbits.CREN = 0;
            RCSTAbits.CREN = 1;
        }
        
        // Octect reçu sur le port série
        while ( !PIR1bits.RCIF );

        for ( int i = 0 ; i < BUFFER_1_SIZE - 1 ; i++ ){
            buffer1[i] = buffer1[i+1];
        }
        buffer1[BUFFER_1_SIZE - 1] = RCREG;
        
        // Si la fin du buffer correspond à un \r, c'est qu'une instruction a été envoyée
        if ( buffer1[BUFFER_1_SIZE - 1] == '\r' ){
            decoderCommande();
        }
    }
    
    return (EXIT_SUCCESS);
}